@@ -103,12 +103,16 @@ namespace xt
         using iterator = typename iterable_base::iterator;
         using const_iterator = typename iterable_base::const_iterator;
 
+        static constexpr layout_type static_layout = xexpression_type::static_layout;
+        static constexpr bool contiguous_layout = xexpression_type::contiguous_layout;
+
         template <class CTA, class S>
         xbroadcast(CTA&& e, S&& s) noexcept;
 
         size_type size() const noexcept;
         size_type dimension() const noexcept;
         const inner_shape_type& shape() const noexcept;
+        layout_type layout() const noexcept;
 
         template <class... Args>
         const_reference operator()(Args... args) const;
@@ -227,6 +231,15 @@ namespace xt
     {
         return m_shape;
     }
+
+    /**
+     * Returns the layout_type of the expression.
+     */
+    template <class CT, class X>
+    inline layout_type xbroadcast<CT, X>::layout() const noexcept
+    {
+        return m_e.layout();
+    }
     //@}
 
     /**
