@@ -19,6 +19,8 @@
 
 #include "xexpression.hpp"
 #include "xiterator.hpp"
+#include "xlayout.hpp"
+#include "xscalar.hpp"
 #include "xutils.hpp"
 
 namespace xt
@@ -128,12 +130,16 @@ namespace xt
         using const_broadcast_iterator = xiterator<const_stepper, shape_type*>;
         using broadcast_iterator = const_broadcast_iterator;
 
+        static constexpr layout_type static_layout = compute_layout(std::decay_t<CT>::static_layout...);
+        static constexpr bool contiguous_layout = and_c<std::decay_t<CT>::contiguous_layout...>::value;
+
         template <class Func>
         xfunction(Func&& f, CT... e) noexcept;
 
         size_type size() const noexcept;
         size_type dimension() const noexcept;
         const shape_type& shape() const;
+        layout_type layout() const noexcept;
 
         template <class... Args>
         const_reference operator()(Args... args) const;
@@ -176,6 +182,9 @@ namespace xt
 
     private:
 
+        template <std::size_t... I>
+        layout_type layout_impl(std::index_sequence<I...>) const noexcept;
+
         template <std::size_t... I, class... Args>
         const_reference access_impl(std::index_sequence<I...>, Args... args) const;
 
@@ -235,7 +244,7 @@ namespace xt
         reference deref_impl(std::index_sequence<I...>) const;
 
         const xfunction_type* p_f;
-        std::tuple<typename std::decay_t<CT>::const_iterator...> m_it;
+        std::tuple<get_iterator<const std::decay_t<CT>>...> m_it;
     };
 
     template <class F, class R, class... CT>
@@ -358,6 +367,15 @@ namespace xt
         }
         return m_shape;
     }
+
+    /**
+     * Returns the layout_type of the xfunction.
+     */
+    template <class F, class R, class... CT>
+    inline layout_type xfunction<F, R, CT...>::layout() const noexcept
+    {
+        return layout_impl(std::make_index_sequence<sizeof...(CT)>());
+    }
     //@}
 
     /**
@@ -445,7 +463,7 @@ namespace xt
     template <class F, class R, class... CT>
     inline auto xfunction<F, R, CT...>::begin() const noexcept -> const_iterator
     {
-        auto f = [](const auto& e) noexcept { return e.cbegin(); };
+        auto f = [](const auto& e) noexcept { return detail::trivial_begin(e); };
         return build_iterator(f, std::make_index_sequence<sizeof...(CT)>());
     }
 
@@ -456,7 +474,7 @@ namespace xt
     template <class F, class R, class... CT>
     inline auto xfunction<F, R, CT...>::end() const noexcept -> const_iterator
     {
-        auto f = [](const auto& e) noexcept { return e.cend(); };
+        auto f = [](const auto& e) noexcept { return detail::trivial_end(e); };
         return build_iterator(f, std::make_index_sequence<sizeof...(CT)>());
     }
 
@@ -588,6 +606,13 @@ namespace xt
         return build_stepper(f, std::make_index_sequence<sizeof...(CT)>());
     }
 
+    template <class F, class R, class... CT>
+    template <std::size_t... I>
+    inline layout_type xfunction<F, R, CT...>::layout_impl(std::index_sequence<I...>) const noexcept
+    {
+        return compute_layout(std::get<I>(m_e).layout()...);
+    }
+
     template <class F, class R, class... CT>
     template <std::size_t... I, class... Args>
     inline auto xfunction<F, R, CT...>::access_impl(std::index_sequence<I...>, Args... args) const -> const_reference
