@@ -10,8 +10,8 @@
 #define XTENSOR_CONFIG_HPP
 
 #define XTENSOR_VERSION_MAJOR 0
-#define XTENSOR_VERSION_MINOR 8
-#define XTENSOR_VERSION_PATCH 4
+#define XTENSOR_VERSION_MINOR 9
+#define XTENSOR_VERSION_PATCH 0
 
 // DETECT 3.6 <= clang < 3.8 for compiler bug workaround.
 #ifdef __clang__
@@ -31,4 +31,8 @@
     std::vector<typename DEFAULT_DATA_CONTAINER(T, EA)::size_type, SA>
 #endif
 
+#ifndef DEFAULT_LAYOUT
+#define DEFAULT_LAYOUT layout_type::row_major
+#endif
+
 #endif
