@@ -104,6 +104,9 @@ namespace xt
         using iterator = typename iterable_base::iterator;
         using const_iterator = typename iterable_base::const_iterator;
 
+        static constexpr layout_type static_layout = layout_type::dynamic;
+        static constexpr bool contiguous_layout = false;
+
         template <class CTA, class... SL>
         explicit xview(CTA&& e, SL&&... slices) noexcept;
 
@@ -118,6 +121,7 @@ namespace xt
         size_type size() const noexcept;
         const inner_shape_type& shape() const noexcept;
         const slice_type& slices() const noexcept;
+        layout_type layout() const noexcept;
 
         template <class... Args>
         reference operator()(Args... args);
@@ -412,6 +416,15 @@ namespace xt
     {
         return m_slices;
     }
+        
+    /**
+     * Returns the slices of the view.
+     */
+    template <class CT, class... S>
+    inline layout_type xview<CT, S...>::layout() const noexcept
+    {
+        return static_layout;
+    }
 
     /**
      * Returns the data holder of the underlying container (only if the view is on a realized
